@@ -5,7 +5,7 @@
 #  include <unistd.h>
 #  include <sys/types.h>
 #endif
-/* last modified 01/07/09 */
+/* last modified 06/07/09 */
 /*
  *******************************************************************************
  *                                                                             *
@@ -16,14 +16,11 @@
  *                                                                             *
  *******************************************************************************
  */
-int Iterate(int wtm, int search_type, int root_list_done)
-{
+int Iterate(int wtm, int search_type, int root_list_done) {
   ROOT_MOVE temp;
-  static int prev_time = 0;
-  int extended = 0, i;
-  register int wpawn, bpawn, TB_use_ok, value = 0, twtm, used;
+  int i;
+  register int value = 0, twtm, used;
   register int correct, correct_count, material = 0, sorted;
-  register unsigned int time_used;
   char *fh_indicator, *fl_indicator;
   TREE *const tree = block[0];
 
@@ -40,8 +37,6 @@ int Iterate(int wtm, int search_type, int root_list_done)
  ************************************************************
  */
   tree->curmv[0] = 0;
-  if (average_nps == 0)
-    average_nps = 150000 * max_threads;
   if (wtm) {
     draw_score[0] = -abs_draw_score;
     draw_score[1] = abs_draw_score;
@@ -67,16 +62,6 @@ int Iterate(int wtm, int search_type, int root_list_done)
   parallel_splits = 0;
   parallel_aborts = 0;
   max_split_blocks = 0;
-  TB_use_ok = 1;
-  if (TotalPieces(white, pawn) && TotalPieces(black, pawn)) {
-    wpawn = MSB(Pawns(white));
-    bpawn = MSB(Pawns(black));
-    if (FileDistance(wpawn, bpawn) == 1) {
-      if (((Rank(wpawn) == RANK2) && (Rank(bpawn) > RANK3)) ||
-          ((Rank(bpawn) == RANK7) && (Rank(wpawn) < RANK6)) || EnPassant(1))
-        TB_use_ok = 0;
-    }
-  }
   if (booking || !Book(tree, wtm, root_list_done))
     do {
       if (abort_search)
@@ -98,10 +83,10 @@ int Iterate(int wtm, int search_type, int root_list_done)
       tree->evaluations = 0;
       tree->egtb_probes = 0;
       tree->egtb_probes_successful = 0;
-      tree->check_extensions_done = 0;
-      tree->qsearch_check_extensions_done = 0;
-      tree->reductions_attempted = 0;
+      tree->extensions_done = 0;
+      tree->qchecks_done = 0;
       tree->reductions_done = 0;
+      tree->moves_pruned = 0;
       root_wtm = wtm;
 /*
  ************************************************************
@@ -116,11 +101,10 @@ int Iterate(int wtm, int search_type, int root_list_done)
         program_end_time = ReadClock();
         tree->pv[0].pathl = 0;
         tree->pv[0].pathd = 0;
-        if (Check(wtm)) {
+        if (Check(wtm))
           root_value = -(MATE - 1);
-        } else {
+        else
           root_value = DrawScore(wtm);
-        }
         Print(6, "              depth   time  score   variation\n");
         Print(6, "                                    (no moves)\n");
         tree->nodes_searched = 1;
@@ -146,7 +130,7 @@ int Iterate(int wtm, int search_type, int root_list_done)
  *                                                          *
  ************************************************************
  */
-      TimeSet(search_type);
+      TimeSet(tree, search_type);
       iteration_depth = 1;
       if (last_pv.pathd > 1)
         iteration_depth = last_pv.pathd + 1;
@@ -182,11 +166,11 @@ int Iterate(int wtm, int search_type, int root_list_done)
  ************************************************************
  */
 #if (CPUS > 1)
-      if (max_threads > smp_idle + 1) {
+      if (smp_max_threads > smp_idle + 1) {
         long proc;
 
         initialized_threads = 1;
-        for (proc = smp_threads + 1; proc < max_threads; proc++) {
+        for (proc = smp_threads + 1; proc < smp_max_threads; proc++) {
           Print(128, "starting thread %d\n", proc);
           thread[proc] = 0;
 #  if defined(_WIN32) || defined(_WIN64)
@@ -223,7 +207,7 @@ int Iterate(int wtm, int search_type, int root_list_done)
             Print(4095, "pathlen=%d\n", tree->pv[0].pathl);
             break;
           }
-          HashStorePV(tree, i, twtm, tree->pv[0].path[i]);
+          HashStorePV(tree, twtm, tree->pv[0].path[i]);
           MakeMove(tree, i, tree->pv[0].path[i], twtm);
           twtm = Flip(twtm);
         }
@@ -271,47 +255,11 @@ int Iterate(int wtm, int search_type, int root_list_done)
             if (!(root_moves[i].status & 256))
               break;
           root_moves[i].status &= 4095 - 128;
-          value = SearchRoot(tree, root_alpha, root_beta, wtm, iteration_depth);
+          value =
+              SearchRoot(tree, root_alpha, root_beta, wtm, iteration_depth);
           root_print_ok = tree->nodes_searched > noise_level;
           if (abort_search || time_abort)
             break;
-/*
- ************************************************************
- *                                                          *
- *   Keep track of the time used for the last ply.  If the  *
- *   next ply doesn't have at least as much time as the     *
- *   previous ply, then it is likely that the last ply will *
- *   be the last complete ply searched, and the next ply    *
- *   will only be a partial search.  In this case, extend   *
- *   the search time giving the next ply the same amount of *
- *   time as the previous ply.  This should be enough time  *
- *   to get a good search on the best move and allow for a  *
- *   fail-low.  If the next ply actually finishes before    *
- *   the timeout, then allow for a second extension (limit  *
- *   2 times) and search again.                             *
- *                                                          *
- ************************************************************
- */
-          if (iteration_depth > 3) {
-            int time_left, prev_ply;
-            time_used = ReadClock() - start_time;
-            time_left = time_limit - time_used;
-            prev_ply = ReadClock() - prev_time;
-
-            if (time_left < prev_ply) {
-              if (extended++ < 2) {
-                time_limit = time_used + prev_ply;
-                if (time_limit > absolute_time_limit)
-                  time_limit = absolute_time_limit;
-              } else
-                time_limit = time_used;
-            }
-          } else
-            extended = 0;
-          prev_time = ReadClock();
-
-          if (time_limit > absolute_time_limit)
-            time_limit = absolute_time_limit;
 /*
  ************************************************************
  *                                                          *
@@ -324,13 +272,12 @@ int Iterate(int wtm, int search_type, int root_list_done)
  ************************************************************
  */
           if (value >= root_beta) {
-            if (!(root_moves[0].status & 8)) {
+            if (!(root_moves[0].status & 8))
               root_moves[0].status |= 8;
-            } else if (!(root_moves[0].status & 16)) {
+            else if (!(root_moves[0].status & 16))
               root_moves[0].status |= 16;
-            } else {
+            else
               root_moves[0].status |= 32;
-            }
             root_alpha = SetRootAlpha(root_moves[0].status, root_alpha);
             root_value = root_alpha;
             root_beta = SetRootBeta(root_moves[0].status, root_beta);
@@ -356,13 +303,14 @@ int Iterate(int wtm, int search_type, int root_list_done)
                 Print(2, "%d. ", move_number);
               if ((display_options & 64) && !wtm)
                 Print(2, "... ");
-              Print(2, "%s!!\n", OutputMove(tree, tree->pv[1].path[1], 1, wtm));
+              Print(2, "%s!                    \n", OutputMove(tree,
+                      tree->pv[1].path[1], 1, wtm));
               kibitz_text[0] = 0;
               if (display_options & 64)
                 sprintf(kibitz_text, " %d.", move_number);
               if ((display_options & 64) && !wtm)
                 sprintf(kibitz_text + strlen(kibitz_text), " ...");
-              sprintf(kibitz_text + strlen(kibitz_text), " %s!!",
+              sprintf(kibitz_text + strlen(kibitz_text), " %s!",
                   OutputMove(tree, tree->pv[1].path[1], 1, wtm));
               Kibitz(6, wtm, iteration_depth, end_time - start_time, value,
                   tree->nodes_searched, tree->egtb_probes_successful,
@@ -381,13 +329,12 @@ int Iterate(int wtm, int search_type, int root_list_done)
  */
           } else if (value <= root_alpha) {
             if (!(root_moves[0].status & 0x38)) {
-              if (!(root_moves[0].status & 1)) {
+              if (!(root_moves[0].status & 1))
                 root_moves[0].status |= 1;
-              } else if (!(root_moves[0].status & 2)) {
+              else if (!(root_moves[0].status & 2))
                 root_moves[0].status |= 2;
-              } else {
+              else
                 root_moves[0].status |= 4;
-              }
               root_alpha = SetRootAlpha(root_moves[0].status, root_alpha);
               root_value = root_alpha;
               root_beta = SetRootBeta(root_moves[0].status, root_beta);
@@ -408,13 +355,15 @@ int Iterate(int wtm, int search_type, int root_list_done)
                   if (root_moves[0].status & 4)
                     fl_indicator = "+M";
                 }
-                Print(4, "               %2i   %s     %2s   ", iteration_depth,
-                    DisplayTime(ReadClock() - start_time), fl_indicator);
+                Print(4, "               %2i   %s     %2s   ",
+                    iteration_depth, DisplayTime(ReadClock() - start_time),
+                    fl_indicator);
                 if (display_options & 64)
                   Print(4, "%d. ", move_number);
                 if ((display_options & 64) && !wtm)
                   Print(4, "... ");
-                Print(4, "%s\n", OutputMove(tree, root_moves[0].move, 1, wtm));
+                Print(4, "%s?                    \n", OutputMove(tree,
+                        root_moves[0].move, 1, wtm));
               }
             } else
               break;
@@ -520,7 +469,8 @@ int Iterate(int wtm, int search_type, int root_list_done)
                 correct_count >= early_exit || value > MATE - 300 ||
                 tree->pv[0].pathh == 2)) {
           if (value != -(MATE - 1))
-            DisplayPV(tree, 5, wtm, end_time - start_time, value, &tree->pv[0]);
+            DisplayPV(tree, 5, wtm, end_time - start_time, value,
+                &tree->pv[0]);
         }
         root_alpha = value - 40;
         root_value = root_alpha;
@@ -538,8 +488,8 @@ int Iterate(int wtm, int search_type, int root_list_done)
         if (correct_count >= early_exit)
           break;
 #if !defined(NOEGTB)
-        if (iteration_depth > 3 && TotalAllPieces <= EGTBlimit && TB_use_ok &&
-            EGTB_use && !EGTB_search && EGTBProbe(tree, 1, wtm, &i))
+        if (iteration_depth > 3 && TotalAllPieces <= EGTBlimit && EGTB_use &&
+            !EGTB_search && EGTBProbe(tree, 1, wtm, &i))
           break;
 #endif
         if (search_nodes && tree->nodes_searched >= search_nodes)
@@ -559,17 +509,11 @@ int Iterate(int wtm, int search_type, int root_list_done)
       if (elapsed_end > 10)
         nodes_per_second =
             (BITBOARD) tree->nodes_searched * 100 / (BITBOARD) elapsed_end;
-      if (!tree->egtb_probes) {
-        if (average_nps)
-          average_nps = (9 * average_nps + nodes_per_second) / 10;
-        else
-          average_nps = nodes_per_second;
-      }
       tree->evaluations = (tree->evaluations) ? tree->evaluations : 1;
       if ((!abort_search || time_abort) && !puzzling) {
         tree->fail_high++;
         tree->fail_high_first++;
-        material = Material / pawn_value;
+        material = Material / PieceValues(white, pawn);
         Print(8, "              time=%s  mat=%d",
             DisplayTimeKibitz(end_time - start_time), material);
         Print(8, "  n=" BMF, tree->nodes_searched);
@@ -577,13 +521,13 @@ int Iterate(int wtm, int search_type, int root_list_done)
             (int) ((BITBOARD) tree->fail_high_first * 100 /
                 (BITBOARD) tree->fail_high));
         Print(8, "  nps=%s\n", DisplayKM(nodes_per_second));
-        Print(16, "              ext-> check=%s ",
-            DisplayKM(tree->check_extensions_done));
-        Print(16, "qcheck=%s ", DisplayKM(tree->qsearch_check_extensions_done));
-        Print(16, "reduce=%s", DisplayKM(tree->reductions_attempted));
-        Print(16, "/%s\n", DisplayKM(tree->reductions_done));
-        Print(16, "              predicted=%d  evals=%s  50move=%d", predicted,
-            DisplayKM(tree->evaluations), Rule50Moves(0));
+        Print(16, "              extensions=%s ",
+            DisplayKM(tree->extensions_done));
+        Print(16, "qchecks=%s ", DisplayKM(tree->qchecks_done));
+        Print(16, "reduced=%s ", DisplayKM(tree->reductions_done));
+        Print(16, "pruned=%s\n", DisplayKM(tree->moves_pruned));
+        Print(16, "              predicted=%d  evals=%s  50move=%d",
+            predicted, DisplayKM(tree->evaluations), Rule50Moves(0));
         Print(16, "  EGTBprobes=%s  hits=%s\n", DisplayKM(tree->egtb_probes),
             DisplayKM(tree->egtb_probes_successful));
         Print(16, "              SMP->  splits=%d  aborts=%d  data=%d/%d  ",
@@ -600,14 +544,18 @@ int Iterate(int wtm, int search_type, int root_list_done)
       Kibitz(4, wtm, 0, 0, 0, 0, 0, kibitz_text);
   }
   if (smp_nice && ponder == 0 && smp_threads) {
-    Print(128, "terminating SMP processes.");
-    quit = 1;
+    int proc;
+
+    Print(128, "terminating SMP processes.\n");
+    for (proc = 1; proc < CPUS; proc++)
+      thread[proc] = (TREE *) - 1;
     while (smp_threads);
-    quit = 0;
     smp_idle = 0;
   }
   program_end_time = ReadClock();
   search_move = 0;
+  if (quit)
+    CraftyExit(0);
   return (last_root_value);
 }
 
@@ -621,8 +569,7 @@ int Iterate(int wtm, int search_type, int root_list_done)
  *                                                                             *
  *******************************************************************************
  */
-int SetRootAlpha(unsigned char status, int old_root_alpha)
-{
+int SetRootAlpha(unsigned char status, int old_root_alpha) {
   if (status & 4)
     return (-MATE - 1);
   if (status & 2)
@@ -642,8 +589,7 @@ int SetRootAlpha(unsigned char status, int old_root_alpha)
  *                                                                             *
  *******************************************************************************
  */
-int SetRootBeta(unsigned char status, int old_root_beta)
-{
+int SetRootBeta(unsigned char status, int old_root_beta) {
   if (status & 32)
     return (MATE + 1);
   if (status & 16)
